//+------------------------------------------------------------------+
//|                                               CyclePeriod_v3.mq4 |
//|                             Copyright  2008-13, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4
// Price    = 7 - Heiken Ashi Close  
// Price    = 8 - Heiken Ashi Open
// Price    = 9 - Heiken Ashi High
// Price    =10 - Heiken Ashi Low


#property copyright "Copyright  2008-13, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  SteelBlue
#property indicator_width1  2
#property indicator_minimum 0

//---- input parameters
extern int     Price        =   4;  //Applied to(see list above) 
extern double  Alpha        =0.07;  //Cycle Smoothing Factor
extern int     MedianPeriod =   5;  //Period of Moving Median
extern int     DCsmooth     =   5;  //Period of Dominant Cycle Smoothing

//---- indicator buffers
double cPeriod[];
double aPrice[];
double DeltaPhase[];

double   haClose[2], haOpen[2], haHigh[2], haLow[2];
double   Smooth[3], Cycle[7], Q1[2],I1[2],iPeriod[2];
datetime prevhatime, prevtime;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   string short_name;
//---- indicator line
   IndicatorBuffers(3);
   SetIndexBuffer(0,   cPeriod); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,    aPrice);   
   SetIndexBuffer(2,DeltaPhase);
   
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
//---- name for DataWindow and indicator subwindow label
   short_name="CyclePeriod_v3("+Price+","+DoubleToStr(Alpha,3)+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"Period");
   
//----
   SetIndexDrawBegin(0,7);
   
   return(0);
}

//+------------------------------------------------------------------+
//| CyclePeriod_v3                                                   |
//+------------------------------------------------------------------+
int start()
{
   int    i, shift, limit, counted_bars=IndicatorCounted();
   
   if(counted_bars < 0) return(0);  
   if(counted_bars > 0) limit=Bars-counted_bars-1;
   if(counted_bars < 1) 
   {
   limit=Bars-1; 
   for(i=0;i<limit;i++) cPeriod[i] = 0;
   }
//----

   for(shift=limit;shift>=0;shift--)
   {	
      if(prevtime != Time[shift])
      {
      for(i=6;i>=1;i--) Cycle[i] = Cycle[i-1];
      Smooth[2] = Smooth[1];
      Smooth[1] = Smooth[0];
      I1[1] = I1[0];
      Q1[1] = Q1[0];
      iPeriod[1] = iPeriod[0];
      prevtime = Time[shift];
      }   

      if(Price <= 6) aPrice[shift] = iMA(NULL,0,1,0,0,Price,shift);   
      else
      if(Price > 6 && Price <= 10) aPrice[shift] = HeikenAshi(Price-7,shift);   
      
   if(shift < Bars - 3) Smooth[0] = (aPrice[shift] + 2*aPrice[shift+1] + 2*aPrice[shift+2] + aPrice[shift+3])/6; 
	Cycle[0] = (1 - 0.5*Alpha)*(1 - 0.5*Alpha)*(Smooth[0] - 2*Smooth[1] + Smooth[2]) + 2*(1-Alpha)*Cycle[1] - (1-Alpha)*(1-Alpha)*Cycle[2];
	if(shift > Bars - 7) Cycle[0] = (aPrice[shift] - 2*aPrice[shift+1] + aPrice[shift+2])/4;  
	
	Q1[0] = (0.0962*Cycle[0] + 0.5769*Cycle[2] - 0.5769*Cycle[4] - 0.0962*Cycle[6])*(0.5+0.08*iPeriod[1]);
   I1[0] = Cycle[3];  
	
	if(Q1[0] != 0 && Q1[1] != 0) DeltaPhase[shift] = (I1[0]/Q1[0] - I1[1]/Q1[1])/(1 + I1[0]*I1[1]/(Q1[0]*Q1[1]));		
	if(DeltaPhase[shift] < 0.1) DeltaPhase[shift] = 0.1;  
	if(DeltaPhase[shift] > 1.1) DeltaPhase[shift] = 1.1;
	double MedianDelta = Median(DeltaPhase,MedianPeriod,shift);

	if(MedianDelta == 0) double DC = 15; else DC = 6.28318/MedianDelta + 0.5;
	
	iPeriod[0]     = 0.33*DC + (1 - 2.0/(DCsmooth + 1))*iPeriod[1];
	cPeriod[shift] = 0.15*iPeriod[0] + 0.85*cPeriod[shift+1];
   }
   
	return(0);	
}


//----
double Median(double price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar+i];
   ArraySort(array);
   
   int num = MathRound(0.5*(per-1)); 
   if(MathMod(per,2)>0) double median = array[num]; else median = 0.5*(array[num]+array[num+1]);
   
   return(median); 
}

// HeikenAshi Price:  7 - Close,8 - Open,9 - High,10 - Low 
double HeikenAshi(int price,int bar)
{ 
   if(prevhatime != Time[bar])
   {
   haClose[1] = haClose[0];
   haOpen [1] = haOpen [0];
   haHigh [1] = haHigh [0];
   haLow  [1] = haLow  [0];
   prevhatime = Time[bar];
   }
   
   if(bar == Bars - 1) 
   {
   haClose[0] = Close[bar];
   haOpen [0] = Open [bar];
   haHigh [0] = High [bar];
   haLow  [0] = Low  [bar];
   }
   else
   {
   haClose[0] = (Open[bar] + High[bar] + Low[bar] + Close[bar])/4;
   haOpen [0] = (haOpen[1] + haClose[1])/2;
   haHigh [0] = MathMax(High[bar],MathMax(haOpen[0],haClose[0]));
   haLow  [0] = MathMin(Low [bar],MathMin(haOpen[0],haClose[0]));
   }
   
   switch(price)
   {
   case 0: return(haClose[0]); break;
   case 1: return(haOpen [0]); break;
   case 2: return(haHigh [0]); break;
   case 3: return(haLow  [0]); break;
   }
}       		